/* HTML5 Elements
   ==============
*/

article, aside, figure, figcaption, footer, header, hgroup, nav, section {
  display: block;
}


/* Search Forms.
   =============
*/
.listing-summary #search-form form span.option {
  float: left;
  margin-right: 0.5em;
}

.listing-summary #search-form form div.field {
  clear: both;
  overflow: hidden;
}

.listing-summary #search-form .form-rows {
  clear: both;
  margin-bottom: 1em;
  overflow: hidden;
  padding: 0 0.25em;
}

.listing-summary #search-form .form-rows .field {
  padding: 0 1%;
  width: 98%;
}

.listing-summary #search-form .form-row-location .field,
.listing-summary #search-form .form-row-price .field,
.listing-summary #search-form .form-row-beds-baths .field,
.listing-summary #search-form .form-row-sizes .field,
.listing-summary #search-form .form-row-other .field {
  clear: none;
  float: left;
  padding: 0 1%;
  width: 31%;
}

.listing-summary #search-form .form-rows .field select,
.listing-summary #search-form .form-rows .field input[type=text] {
  width: 90%;
}

.listing-summary #search-form .form-row-beds-baths .field select,
.listing-summary #search-form .form-row-sizes .field select {
  width: 40%;
}

.listing-summary #search-form .form-row-beds-baths .field .separator,
.listing-summary #search-form .form-row-sizes .field .separator {
  display: inline-block;
  text-align: center;
  width: 10%;
}

#content .listing-summary #search-form dd {
  float: none;
  width: auto;
}


/* Listing Detail page.
   ====================
*/
.listing #listing-info {
  font-size: 0.9em;
  margin-bottom: 1.5em;
}

.listing #listing-info dl {
  border-bottom: 1px dotted #CCC;
  margin: 0.5em 0;
  overflow: hidden;
  padding: 0.5em 0;
  word-wrap: break-word;
}

.listing #listing-info dt {
  clear: both;
  display: inline;
  float: left;
  margin-right: 5%;
  width: 35%;
}

.listing #listing-info dd {
  display: inline;
  float: left;
  margin-bottom: 0;
  margin-left: 0;
  margin-right: 0;
  width: 60%;
}

.listing #listing-images {
  float: left;
  margin-bottom: 1.5em;
  margin-right: 1.5em;
  width: 412px;
  max-width:100%;
}

.listing #listing-images img{
  max-width: 100%;
}

.listing #listing-images .galleria-container {
  background: transparent;
}

.listing #listing-details,
.listing ul.formTabs {
  clear: both;
}

.listing a#agentquote {
  background: transparent url('++resource++plone.mls.listing.images/icon_talk.png') no-repeat 0% 50%;
  border: none;
  color: #999;
  display: inline-block;
  line-height: 2em;
  padding-left: 3em;
}

.listing #sec-agent-quote {
  background: #f9f7ee url('++resource++plone.mls.listing.images/quotation_left.png') no-repeat 0.5em 1.5em;
  border: 2px solid #f2f1f1;
  margin-bottom: 2em;
  padding: 2em 0 1em 2em;
}

.listing #sec-agent-quote h2 {
  border-bottom: 1px dotted #666;
  line-height: 1.25em;
  margin-left: 1em;
  margin-right: 2.5em;
  padding-bottom: 0.5em;
}

.listing #sec-agent-quote blockquote {
  color: #666;
  font-style: normal;
  margin: 0 3.5em 1em 1.5em;
}

.listing #sec-listing-long-description {
  background: transparent url('++resource++plone.mls.listing.images/quotation_right.png') no-repeat 100% 100%;
  margin-left: 1.5em;
}

.listing #sec-listing-long-description h3 {
  border-top: 1px dotted #666;
  font-size: 1.25em;
  margin-bottom: 0.25em;
  margin-right: 3.5em;
  padding-top: 1em;
}

.listing #sec-listing-long-description p {
  margin-bottom: 0;
  margin-right: 3.5em;
  text-align: justify;
}



/* Table-like listing summary.
   ===========================
*/
#content .listing-summary .tileItem {
  margin-bottom: 1em;
}

#content .listing-summary figure {
  float: left;
  margin-right: 3%;
  width: 22%;
}

#content .listing-summary figure a {
  border: 1px solid #D5D5D5;
  display: block;
  padding: 3%;
}

#content .listing-summary figure img {
  width: 100%;
}

#content .listing-summary figure figcaption {
  display: none;
}

#content .listing-summary section {
  float: left;
  width: 75%;
}

#content .listing-summary dt {
  float: left;
  margin: 0 3% 0 0;
  width: 22%;
  word-wrap: break-word;
}

#content .listing-summary dt.row {
  clear: both;
}

#content .listing-summary dd {
  float: left;
  margin: 0 3% 0 0;
  width: 22%;
}

#content .listing-summary dd.full {
  float: left;
  margin: 0;
  width: 75%;
}

/* AgentInformation Portlet
   ========================
*/
.portletAgentInformation .portletItem figure img {
  max-height: 100%;
  max-width: 100%;
}

.portletAgentInformation .portletItem figure figcaption {
  display: none;
}

.portletAgentInformation .portletItem .agent_title,
.portletAgentInformation .portletItem .agency_name {
  font-weight: bold;
  margin-bottom: 0;
}

.portletAgentInformation .portletItem .contact_details {
  margin-top: 1em;
  word-wrap: break-word;
}

.portletAgentInformation .portletItem .contact_details dt {
  background: transparent;
  clear: both;
  float: left;
  margin: 0 3% 0 0;
  padding: 0.42em 0;
  width: 37%;
}

.portletAgentInformation .portletItem .contact_details dd {
  float: left;
  margin: 0;
  padding: 0.42em 0;
  width: 60%;
}

/* Video Embedding Listing Details */
.listing-video{
  margin-top:1em;
  margin-bottom:2em;
  position: relative;
  padding-bottom: 56.25%; /* 16:9 */
  padding-top: 25px;
  height: 0;
}

.listing-video iframe {
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
}

/* Google Maps style */
.map__canvas {
    height: 340px;
    width: 100%;
}
