@page {
  margin-left: 2cm;
  margin-right: 2cm;
}

body {
  font: 12pt "Times New Roman", Times, serif;
  line-height: 1.3;
}

h1 {
  font-size: 24pt;
}

h2 {
  font-size: 16pt;
  margin-top: 25px;
  margin-bottom: 5px;
}

h3 {
  font-size: 12pt;
}

img {
  max-width: 275px;
}

#listing-info img {
  float: right;
  max-width: 25%;
}

#listing-info dl,
.contact-information dl {
  width: 75%;
}

#listing-details,
#listing-images {
  page-break-before: always;
}

#listing-images ul {
  list-style: none;
  padding: 0;
}

#listing-images li {
  float: left;
  margin: 0.5em;
}


dl {
  float: left;
  margin-top: 0;
  width: 100%;
}

dt {
  clear: both;
  display: inline;
  float: left;
  font-weight: bold;
  margin-right: 5%;
  width: 35%;
}

dd {
  display: inline;
  float: left;
  margin-bottom: 0;
  margin-left: 0;
  margin-right: 0;
  width: 60%;
}

blockquote {
  margin: 1em 0;
  text-align: justify;
}

.visualClear {
  clear: both;
}